﻿Option Strict On
Imports Microsoft.Kinect
Imports RingBufferLib.Kinect_RingBuffer
Imports Microsoft.Speech.Recognition '音声認識をするためのクラスが含まれる。
Imports Microsoft.Speech.AudioFormat '音声認識のオーディオ形式を表わすクラスが含まれる。
Imports System.IO
Class MainWindow

    Dim Kinect As KinectSensor

    Dim myScreenImageRect As Int32Rect
    Dim myScreenImageRect2 As Int32Rect

    Dim myDepthPixelData As Short()
    Dim myColorPixelData As Byte()

    Dim targetDepth As Double

    Dim BytesPerPixel As Integer = CInt(PixelFormats.Bgr32.BitsPerPixel / 8)
    Dim maxPlayer As Integer = 0

    Const bufferSecond As Integer = 3

    Dim screenImageStride As Integer

    Dim myImageSize As Int32Rect
    Dim myRingBuffer As RingBuffer

    Dim myOverrayBitmap As WriteableBitmap
    Dim engine As SpeechRecognitionEngine
    Dim words As String
    Dim speechNo As Integer = 1

    Property Room_Bitmap As WriteableBitmap

    Private Sub MainWindow_Loaded(sender As Object, e As System.Windows.RoutedEventArgs) Handles Me.Loaded
        speechNo = 1
        targetDepth = 2000 '2400
      
        init_kinect()

        AddHandler CompositionTarget.Rendering, Sub(renderSender As Object, renderArgs As EventArgs)
                                                    Using colorFrame As ColorImageFrame = Kinect.ColorStream.OpenNextFrame(100)
                                                        Using depthFrame As DepthImageFrame = Kinect.DepthStream.OpenNextFrame(100)
                                                            Using mySkeletonFrame As SkeletonFrame = Kinect.SkeletonStream.OpenNextFrame(100)
                                                                maxPlayer = speechNo
                                                                RenderScreen(colorFrame, depthFrame)
                                                                SaveBuffer(colorFrame, depthFrame, mySkeletonFrame)
                                                            End Using

                                                        End Using
                                                    End Using

                                                End Sub

        DataContext = Me

        Dim sentence As Choices = New Choices
        With sentence
            .Add("分身")
            .Add("統合")
            .Add("おわり")
        End With

        Dim builder As GrammarBuilder = New GrammarBuilder
        builder.Append(sentence)
        Dim myGrammer As Grammar = New Grammar(builder)
        engine = New SpeechRecognitionEngine()
        engine.LoadGrammar(myGrammer)


        AddHandler engine.SpeechRecognized, Sub(speechSender As Object, speechArgs As SpeechRecognizedEventArgs)
                                                Dim myConfidence As Single = speechArgs.Result.Confidence
                                                If myConfidence > 0.3 Then
                                                    words = speechArgs.Result.Text

                                                    Select Case words
                                                        Case "おわり"
                                                            If Kinect Is Nothing = False Then
                                                                If Kinect.IsRunning = True Then
                                                                    Kinect.Stop()
                                                                    Kinect.AudioSource.Stop()
                                                                    engine.RecognizeAsyncStop()
                                                                End If
                                                                Environment.Exit(0)
                                                            End If

                                                        Case "分身"
                                                            Using colorFrame As ColorImageFrame = Kinect.ColorStream.OpenNextFrame(100)
                                                                Using depthFrame As DepthImageFrame = Kinect.DepthStream.OpenNextFrame(100)
                                                                    Using mySkeletonFrame As SkeletonFrame = Kinect.SkeletonStream.OpenNextFrame(100)
                                                                        If speechNo >= 6 Then
                                                                            TextBlock1.Text = "これ以上は分身できません。"
                                                                            Exit Sub
                                                                        Else
                                                                            speechNo = speechNo + 1
                                                                        End If
                                                                        RenderScreen(colorFrame, depthFrame)
                                                                        SaveBuffer(colorFrame, depthFrame, mySkeletonFrame)
                                                                    End Using

                                                                End Using
                                                            End Using

                                                        Case "統合"
                                                            Using colorFrame As ColorImageFrame = Kinect.ColorStream.OpenNextFrame(100)
                                                                Using depthFrame As DepthImageFrame = Kinect.DepthStream.OpenNextFrame(100)
                                                                    Using mySkeletonFrame As SkeletonFrame = Kinect.SkeletonStream.OpenNextFrame(100)
                                                                        TextBlock1.Text = String.Empty
                                                                        speechNo = 1
                                                                        RenderScreen(colorFrame, depthFrame)
                                                                        SaveBuffer(colorFrame, depthFrame, mySkeletonFrame)
                                                                    End Using

                                                                End Using
                                                            End Using
                                                        Case Else
                                                            Exit Select
                                                    End Select
                                                End If
                                            End Sub

        Dim audio As KinectAudioSource = Kinect.AudioSource

        Using s As Stream = audio.Start()
            engine.SetInputToDefaultAudioDevice()
            engine.RecognizeAsync(RecognizeMode.Multiple)
        End Using
    End Sub

    Private Sub init_kinect()
        Try
            If KinectSensor.KinectSensors.Count = 0 Then
                MessageBox.Show("KINECTが接続されておりません。")
                Exit Sub
            Else
                Kinect = KinectSensor.KinectSensors(0)
            End If


            Kinect.DepthStream.Enable(DepthImageFormat.Resolution320x240Fps30) '640×480では重くて動かない。
            Kinect.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30)
            Kinect.SkeletonStream.Enable()

            Dim depthStream As DepthImageStream = Kinect.DepthStream

            myDepthPixelData = New Short(Kinect.DepthStream.FramePixelDataLength - 1) {}
            myColorPixelData = New Byte(Kinect.ColorStream.FramePixelDataLength - 1) {}

            myOverrayBitmap = New WriteableBitmap(depthStream.FrameWidth, depthStream.FrameHeight, 96, 96, PixelFormats.Bgra32, Nothing)
            myScreenImageRect = New Int32Rect(0, 0, CInt(Math.Ceiling(myOverrayBitmap.Width)), CInt(Math.Ceiling(myOverrayBitmap.Height)))

            myRingBuffer.init_ringbuffer(bufferSecond, Kinect.ColorStream.FramePixelDataLength, Kinect.DepthStream.FramePixelDataLength)

            Room_Bitmap = New WriteableBitmap(depthStream.FrameWidth, depthStream.FrameHeight, 96, 96, PixelFormats.Bgra32, Nothing)
            myScreenImageRect2 = New Int32Rect(0, 0, CInt(Math.Ceiling(Room_Bitmap.Width)), CInt(Math.Ceiling(Room_Bitmap.Height)))

            Kinect.Start()

            myImageSize.Width = Kinect.ColorStream.FrameWidth
            myImageSize.Height = Kinect.ColorStream.FrameHeight
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Exit Sub
        End Try
    End Sub


    Private Sub RenderScreen(colorFrame As ColorImageFrame, depthFrame As DepthImageFrame)
        If Kinect Is Nothing OrElse depthFrame Is Nothing OrElse colorFrame Is Nothing Then
            Return
        End If

        Dim depthPixelIndex As Integer
        Dim playerIndex As Integer
        Dim colorPixelIndex As Integer
        Dim colorPoint As ColorImagePoint
        Dim colorStride As Integer = colorFrame.Width * colorFrame.BytesPerPixel
        screenImageStride = depthFrame.Width * colorFrame.BytesPerPixel

        Dim ImageIndex As Integer = 0
        Dim bytePlayer2 As Byte() = New Byte(depthFrame.Height * screenImageStride - 1) {}
        Dim bytePlayer1 As Byte() = New Byte(depthFrame.Height * screenImageStride - 1) {}
        Dim byteRoom As Byte() = New Byte(depthFrame.Height * screenImageStride - 1) {}

        depthFrame.CopyPixelDataTo(myDepthPixelData)
        colorFrame.CopyPixelDataTo(myColorPixelData)

        For depthY As Integer = 0 To depthFrame.Height - 1
            Dim depthX As Integer = 0
            While depthX < depthFrame.Width
                depthPixelIndex = depthX + (depthY * depthFrame.Width)
                playerIndex = myDepthPixelData(depthPixelIndex) And DepthImageFrame.PlayerIndexBitmask

                colorPoint = Kinect.MapDepthToColorImagePoint(depthFrame.Format, depthX, depthY, myDepthPixelData(depthPixelIndex), colorFrame.Format)
                colorPixelIndex = (colorPoint.X * colorFrame.BytesPerPixel) + (colorPoint.Y * colorStride)

                If playerIndex = 0 Then
                    byteRoom(ImageIndex) = myColorPixelData(colorPixelIndex)
                    byteRoom(ImageIndex + 1) = myColorPixelData(colorPixelIndex + 1)
                    byteRoom(ImageIndex + 2) = myColorPixelData(colorPixelIndex + 2)
                    byteRoom(ImageIndex + 3) = &HFF
                End If
                depthX = depthX + 1
                ImageIndex = ImageIndex + colorFrame.BytesPerPixel
            End While
        Next

        Room_Bitmap.WritePixels(myScreenImageRect2, byteRoom, screenImageStride, 0)
    End Sub


    Private Sub SaveBuffer(colorFrame As ColorImageFrame, depthFrame As DepthImageFrame, skeletonFrame As SkeletonFrame)
        If Kinect Is Nothing OrElse depthFrame Is Nothing OrElse colorFrame Is Nothing OrElse skeletonFrame Is Nothing Then
            Return
        End If

        Dim colorStream As ColorImageStream = Kinect.ColorStream
        Dim depthStream As DepthImageStream = Kinect.DepthStream


        Dim colorStride As Integer = colorFrame.BytesPerPixel * colorFrame.Width

        Dim ImageIndex As Integer = 0

        depthFrame.CopyPixelDataTo(myDepthPixelData)
        colorFrame.CopyPixelDataTo(myColorPixelData)


        Dim colorPoint As ColorImagePoint() = New ColorImagePoint(depthFrame.PixelDataLength - 1) {}
        Dim depthPixel As Short() = New Short(depthFrame.PixelDataLength - 1) {}
        Kinect.MapDepthFrameToColorFrame(depthFrame.Format, depthPixel, colorFrame.Format, colorPoint) 'エラー



        Dim bytePlayer As Byte() = New Byte(depthFrame.Height * screenImageStride - 1) {}


        Dim depth As Double() = New Double(depthFrame.Height * screenImageStride - 1) {}
        Dim playerIndexArray As Integer() = New Integer(depthFrame.Height * screenImageStride - 1) {}


        For depthY As Integer = 0 To depthFrame.Height - 1
            Dim depthX As Integer = 0
            While depthX < depthFrame.Width
                Dim depthPixelIndex As Integer = depthX + (depthY * depthFrame.Width)

                Dim playerIndex As Integer = myDepthPixelData(depthPixelIndex) And DepthImageFrame.PlayerIndexBitmask

                Dim x As Integer = colorPoint(depthPixelIndex).X
                Dim y As Integer = colorPoint(depthPixelIndex).Y

                Dim colorPixelIndex As Integer = (x * colorFrame.BytesPerPixel) + (y * colorStride)

                If playerIndex <> 0 Then
                    depth(ImageIndex) = myDepthPixelData(depthPixelIndex) >> DepthImageFrame.PlayerIndexBitmaskWidth
                    playerIndexArray(ImageIndex) = playerIndex

                    bytePlayer(ImageIndex) = myColorPixelData(colorPixelIndex)
                    bytePlayer(ImageIndex + 1) = myColorPixelData(colorPixelIndex + 1)
                    bytePlayer(ImageIndex + 2) = myColorPixelData(colorPixelIndex + 2)
                    bytePlayer(ImageIndex + 3) = &HFF
                End If
                depthX = depthX + 1
                ImageIndex = ImageIndex + colorFrame.BytesPerPixel
            End While
        Next

        myRingBuffer.save_framedata(bytePlayer)
        myRingBuffer.save_playerIndexdata(playerIndexArray)
        myRingBuffer.set_nextframe()
        RenderScreen2()

    End Sub

    Private Sub RenderScreen2()
        If screenImageStride = 0 Then
            Return
        End If
        maxPlayer = speechNo
        Dim bytePlayer As Byte() = New Byte(CInt(myImageSize.Height) * screenImageStride - 1) {}

        Dim frame As Integer = 15

        For i As Integer = 0 To maxPlayer - 1
            Dim iframe As Integer

            If i = 0 Then
                iframe = myRingBuffer.buffer_index - 1  '1人余分な人物が表示されるので-1をする
            Else
                iframe = myRingBuffer.buffer_index + (frame * i)
            End If

            Dim colorPixelData = myRingBuffer.get_rgb_frame(iframe)
            Dim PlayerIndex = myRingBuffer.get_PlayerIndexData(iframe)
            If colorPixelData Is Nothing Then
                Continue For
            End If

            Dim index As Integer = 0
            While index < colorPixelData.Length
                If PlayerIndex(index) > 0 Then
                    bytePlayer(index) = colorPixelData(index)
                    bytePlayer(index + 1) = colorPixelData(index + 1)
                    bytePlayer(index + 2) = colorPixelData(index + 2)
                    bytePlayer(index + 3) = &HFF
                End If
                index = index + BytesPerPixel
            End While
        Next
        myOverrayBitmap.WritePixels(myScreenImageRect, bytePlayer, screenImageStride, 0)
        personImage.Source = myOverrayBitmap
    End Sub


    Private Sub MainWindow_Closing(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles Me.Closing
        If Kinect Is Nothing = False Then
            If Kinect.IsRunning = True Then
                engine.RecognizeAsyncStop()
                Kinect.Stop()
                Kinect.Dispose()
            End If
        End If
    End Sub
End Class

